@@ -6,6 +6,7 @@
 #' @param resamp_res The new resampling resolution.
 #' @param verbose Give occasional updates? Default: \code{FALSE}.
 #' 
+#' @return The resampled \code{"template.cifti"} object.
 #' @export
 #' 
 resample_template <- function(x, resamp_res, verbose=FALSE){
@@ -14,11 +15,11 @@ resample_template <- function(x, resamp_res, verbose=FALSE){
   # Resample the data.
   if (verbose) { cat("Resampling templates.\n") }
   x$template <- lapply(x$template, function(y){
-    as.matrix(resample_xifti(newdata_xifti(x$dat_struct, y), resamp_res=resamp_res, verbose=FALSE))
+    as.matrix(ciftiTools::resample_xifti(ciftiTools::newdata_xifti(x$dat_struct, y), resamp_res=resamp_res, verbose=FALSE))
   })
   if (verbose) { cat("Resampling variance decomposition.\n") }
   x$var_decomp <- lapply(x$var_decomp, function(y){
-    as.matrix(resample_xifti(newdata_xifti(x$dat_struct, y), resamp_res=resamp_res, verbose=FALSE))
+    as.matrix(ciftiTools::resample_xifti(ciftiTools::newdata_xifti(x$dat_struct, y), resamp_res=resamp_res, verbose=FALSE))
   })
 
   if (verbose) { cat("Formatting new template object.\n") }
@@ -27,7 +28,7 @@ resample_template <- function(x, resamp_res, verbose=FALSE){
   x$var_decomp <- lapply(x$var_decomp, function(y){y[] <- ifelse(is.nan(y), NA, y)})
 
   # Get new `dat_struct` and mask.
-  x$dat_struct <- resample_xifti(x$dat_struct, resamp_res=resamp_res)
+  x$dat_struct <- ciftiTools::resample_xifti(x$dat_struct, resamp_res=resamp_res)
   x$mask <- !is.na(x$template$mean[,1])
 
   x
